@@ -1,5 +1,5 @@
 /******************************************************************************
-    Copyright (C) 2013 by Hugh Bailey <obs.jim@gmail.com>
+    Copyright (C) 2023 by Lain Bailey <lain@obsproject.com>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,8 +22,7 @@ static inline bool init_ib(struct gs_index_buffer *ib)
 	GLenum usage = ib->dynamic ? GL_DYNAMIC_DRAW : GL_STATIC_DRAW;
 	bool success;
 
-	success = gl_create_buffer(GL_ELEMENT_ARRAY_BUFFER, &ib->buffer,
-			ib->size, ib->data, usage);
+	success = gl_create_buffer(GL_ELEMENT_ARRAY_BUFFER, &ib->buffer, ib->size, ib->data, usage);
 
 	if (!ib->dynamic) {
 		bfree(ib->data);
@@ -33,22 +32,20 @@ static inline bool init_ib(struct gs_index_buffer *ib)
 	return success;
 }
 
-gs_indexbuffer_t *device_indexbuffer_create(gs_device_t *device,
-		enum gs_index_type type, void *indices, size_t num,
-		uint32_t flags)
+gs_indexbuffer_t *device_indexbuffer_create(gs_device_t *device, enum gs_index_type type, void *indices, size_t num,
+					    uint32_t flags)
 {
 	struct gs_index_buffer *ib = bzalloc(sizeof(struct gs_index_buffer));
-	size_t width = type == GS_UNSIGNED_LONG ? sizeof(long) : sizeof(short);
+	size_t width = type == GS_UNSIGNED_LONG ? 4 : 2;
 
-	ib->device  = device;
-	ib->data    = indices;
+	ib->device = device;
+	ib->data = indices;
 	ib->dynamic = flags & GS_DYNAMIC;
-	ib->num     = num;
-	ib->width   = width;
-	ib->size    = width * num;
-	ib->type    = type;
-	ib->gl_type = type == GS_UNSIGNED_LONG ? GL_UNSIGNED_INT :
-	                                         GL_UNSIGNED_SHORT;
+	ib->num = num;
+	ib->width = width;
+	ib->size = width * num;
+	ib->type = type;
+	ib->gl_type = type == GS_UNSIGNED_LONG ? GL_UNSIGNED_INT : GL_UNSIGNED_SHORT;
 
 	if (!init_ib(ib)) {
 		blog(LOG_ERROR, "device_indexbuffer_create (GL) failed");
@@ -70,15 +67,14 @@ void gs_indexbuffer_destroy(gs_indexbuffer_t *ib)
 	}
 }
 
-void gs_indexbuffer_flush(gs_indexbuffer_t *ib)
+static inline void gs_indexbuffer_flush_internal(gs_indexbuffer_t *ib, const void *data)
 {
 	if (!ib->dynamic) {
 		blog(LOG_ERROR, "Index buffer is not dynamic");
 		goto fail;
 	}
 
-	if (!update_buffer(GL_ELEMENT_ARRAY_BUFFER, ib->buffer, ib->data,
-				ib->size))
+	if (!update_buffer(GL_ELEMENT_ARRAY_BUFFER, ib->buffer, data, ib->size))
 		goto fail;
 
 	return;
@@ -87,6 +83,16 @@ fail:
 	blog(LOG_ERROR, "gs_indexbuffer_flush (GL) failed");
 }
 
+void gs_indexbuffer_flush(gs_indexbuffer_t *ib)
+{
+	gs_indexbuffer_flush_internal(ib, ib->data);
+}
+
+void gs_indexbuffer_flush_direct(gs_indexbuffer_t *ib, const void *data)
+{
+	gs_indexbuffer_flush_internal(ib, data);
+}
+
 void *gs_indexbuffer_get_data(const gs_indexbuffer_t *ib)
 {
 	return ib->data;
