@@ -17,11 +17,16 @@
 
 #pragma once
 
-#if defined(_MSC_VER) && (defined(_M_X64) || defined(_M_IX86)) && \
-	!(defined(_M_ARM64) || defined(_M_ARM64EC))
+#include "c99defs.h"
 
+#if (defined(_MSC_VER) || defined(__MINGW32__)) && ((defined(_M_X64) && !defined(_M_ARM64EC)) || defined(_M_IX86))
 #include <emmintrin.h>
 #else
+#ifndef WIN32_LEAN_AND_MEAN
+#define WIN32_LEAN_AND_MEAN
+#endif
 #define SIMDE_ENABLE_NATIVE_ALIASES
+PRAGMA_WARN_PUSH
 #include "simde/x86/sse2.h"
+PRAGMA_WARN_POP
 #endif
